@@ -3,22 +3,21 @@ Low-level classes for interacting directly with the Strava API webservers.
 """
 from __future__ import division, absolute_import, print_function, unicode_literals
 import abc
-import os.path
 import logging
-import urlparse
 import functools
-from urllib import urlencode
 
 import requests
+import six
+from six.moves.urllib.parse import urlunsplit, urljoin, urlencode
 
 from stravalib import exc
 
 
+@six.add_metaclass(abc.ABCMeta)
 class ApiV3(object):
     """
     This class is responsible for performing the HTTP requests, rate limiting, and error handling.
     """
-    __metaclass__ = abc.ABCMeta
 
     server = 'www.strava.com'
     api_base = '/api/v3'
@@ -29,7 +28,10 @@ class ApiV3(object):
         object.
 
         :param access_token: The token that provides access to a specific Strava account.
+        :type access_token: str
+
         :param requests_session: An existing :class:`requests.Session` object to use.
+        :type requests_session::class:`requests.Session`
         """
         self.log = logging.getLogger('{0.__module__}.{0.__name__}'.format(self.__class__))
         self.access_token = access_token
@@ -41,7 +43,7 @@ class ApiV3(object):
         if rate_limiter is None:
             # Make it a dummy function, so we don't have to check if it's defined before
             # calling it later
-            rate_limiter = lambda: None
+            rate_limiter = lambda x=None: None
 
         self.rate_limiter = rate_limiter
 
@@ -49,7 +51,7 @@ class ApiV3(object):
         """
         Get the URL needed to authorize your application to access a Strava user's information.
 
-        See http://strava.github.io/api/v3/oauth/
+        See https://developers.strava.com/docs/authentication/
 
         :param client_id: The numeric developer client id.
         :type client_id: int
@@ -61,8 +63,10 @@ class ApiV3(object):
                                 Choices are 'auto' or 'force'.  (Default is 'auto')
         :type approval_prompt: str
 
-        :param scope: The access scope required.  Omit to imply "public".  Valid values are 'public', 'write', 'view_private', 'view_private,write'
-        :type scope: str
+        :param scope: The access scope required.  Omit to imply "read" and "activity:read"
+                      Valid values are 'read', 'read_all', 'profile:read_all', 'profile:write', 'profile:read_all',
+                      'activity:read_all', 'activity:write'.
+        :type scope: list[str]
 
         :param state: An arbitrary variable that will be returned to your application in the redirect URI.
         :type state: str
@@ -71,8 +75,21 @@ class ApiV3(object):
         :rtype: str
         """
         assert approval_prompt in ('auto', 'force')
+        if scope is None:
+            scope = ['read', 'activity:read']
+        elif isinstance(scope, (six.text_type, six.binary_type)):
+            scope = [scope]
+
+        unsupported = set(scope) - {'read', 'read_all',
+                                    'profile:read_all', 'profile:write',
+                                    'activity:read', 'activity:read_all',
+                                    'activity:write'}
+
+        assert not unsupported, 'Unsupported scope value(s): {}'.format(unsupported)
+
         if isinstance(scope, (list, tuple)):
             scope = ','.join(scope)
+
         params = {'client_id': client_id,
                   'redirect_uri': redirect_uri,
                   'approval_prompt': approval_prompt,
@@ -82,12 +99,12 @@ class ApiV3(object):
         if state is not None:
             params['state'] = state
 
-        return urlparse.urlunsplit(('https', self.server, '/oauth/authorize', urlencode(params), ''))
+        return urlunsplit(('https', self.server, '/oauth/authorize', urlencode(params), ''))
 
     def exchange_code_for_token(self, client_id, client_secret, code):
         """
         Exchange the temporary authorization code (returned with redirect from strava authorization URL)
-        for a permanent access token.
+        for a short-lived access token and a refresh token (used to obtain the next access token later on).
 
         :param client_id: The numeric developer client id.
         :type client_id: int
@@ -98,23 +115,79 @@ class ApiV3(object):
         :param code: The temporary authorization code
         :type code: str
 
-        :return: The access token.
-        :rtype: str
+        :return: Dictionary containing the access_token, refresh_token
+                 and expires_at (number of seconds since Epoch when the provided access token will expire)
+        :rtype: dict
         """
         response = self._request('https://{0}/oauth/token'.format(self.server),
-                                 params={'client_id': client_id, 'client_secret': client_secret, 'code': code},
+                                 params={'client_id': client_id, 'client_secret': client_secret, 'code': code,
+                                 'grant_type': 'authorization_code'},
                                  method='POST')
-        token = response['access_token']
-        self.access_token = token
-        return token
+        access_info = dict()
+        access_info['access_token'] = response['access_token']
+        access_info['refresh_token'] = response.get('refresh_token', None)
+        access_info['expires_at'] = response.get('expires_at', None)
+        self.access_token = response['access_token']
+
+        return access_info
+
+    def refresh_access_token(self, client_id, client_secret, refresh_token):
+        """
+        Exchanges the previous refresh token for a short-lived access token and a new
+        refresh token (used to obtain the next access token later on).
+
+        :param client_id: The numeric developer client id.
+        :type client_id: int
+
+        :param client_secret: The developer client secret
+        :type client_secret: str
+
+        :param refresh_token: The refresh token obtain from a previous authorization request
+        :type refresh_token: str
+
+        :return: Dictionary containing the access_token, refresh_token
+                 and expires_at (number of seconds since Epoch when the provided access token will expire)
+        :rtype: dict
+        """
+        response = self._request('https://{0}/oauth/token'.format(self.server),
+                                 params={'client_id': client_id, 'client_secret': client_secret,
+                                 'refresh_token': refresh_token, 'grant_type': 'refresh_token'},
+                                 method='POST')
+        access_info = dict()
+        access_info['access_token'] = response['access_token']
+        access_info['refresh_token'] = response['refresh_token']
+        access_info['expires_at'] = response['expires_at']
+        self.access_token = response['access_token']
+
+        return access_info
 
     def _resolve_url(self, url):
         if not url.startswith('http'):
-            url = urlparse.urljoin('https://{0}'.format(self.server), self.api_base + '/' + url.strip('/'))
+            url = urljoin('https://{0}'.format(self.server), self.api_base + '/' + url.strip('/'))
         return url
 
     def _request(self, url, params=None, files=None, method='GET', check_for_errors=True):
+        """
+        Perform the underlying request, returning the parsed JSON results.
 
+        :param url: The request URL.
+        :type url: str
+
+        :param params: Request parameters
+        :type params: Dict[str,Any]
+
+        :param files: Dictionary of file name to file-like objects.
+        :type files: Dict[str,file]
+
+        :param method: The request method (GET/POST/etc.)
+        :type method: str
+
+        :param check_for_errors: Whether to raise
+        :type check_for_errors: bool
+
+        :return: The parsed JSON response.
+        :rtype: Dict[str,Any]
+        """
         url = self._resolve_url(url)
         self.log.info("{method} {url!r} with params {params!r}".format(method=method, url=url, params=params))
         if params is None:
@@ -133,24 +206,19 @@ class ApiV3(object):
             raise ValueError("Invalid/unsupported request method specified: {0}".format(method))
 
         raw = requester(url, params=params)
+        # Rate limits are taken from HTTP response headers
+        # https://strava.github.io/api/#rate-limiting
+        self.rate_limiter(raw.headers)
+
         if check_for_errors:
             self._handle_protocol_error(raw)
 
         # 204 = No content
         if raw.status_code in [204]:
-            resp = []
+            resp = {}
         else:
             resp = raw.json()
 
-        # TODO: We should parse the response to get the rate limit details and
-        # update our rate limiter.
-        # see: http://strava.github.io/api/#access
-
-        # At this stage we should assume that request was successful and we should invoke
-        # our rate limiter.  (Note that this may need to be reviewed; some failures may
-        # also count toward the limit?)
-        self.rate_limiter()
-
         return resp
 
     def _handle_protocol_error(self, response):
@@ -170,16 +238,27 @@ class ApiV3(object):
             if 'message' in json_response or 'errors' in json_response:
                 error_str = '{0}: {1}'.format(json_response.get('message', 'Undefined error'), json_response.get('errors'))
 
-        x = None
-        if 400 <= response.status_code < 500:
-            x = requests.exceptions.HTTPError('%s Client Error: %s [%s]' % (response.status_code, response.reason, error_str))
+        # Special subclasses for some errors
+        msg = None
+        exc_class = None
+        if response.status_code == 404:
+            msg = '%s: %s' % (response.reason, error_str)
+            exc_class = exc.ObjectNotFound
+        elif response.status_code == 401:
+            msg = '%s: %s' % (response.reason, error_str)
+            exc_class = exc.AccessUnauthorized
+        elif 400 <= response.status_code < 500:
+            msg = '%s Client Error: %s [%s]' % (response.status_code, response.reason, error_str)
+            exc_class = exc.Fault
         elif 500 <= response.status_code < 600:
-            x = requests.exceptions.HTTPError('%s Server Error: %s [%s]' % (response.status_code, response.reason, error_str))
+            msg = '%s Server Error: %s [%s]' % (response.status_code, response.reason, error_str)
+            exc_class = exc.Fault
         elif error_str:
-            x = exc.Fault(error_str)
+            msg = error_str
+            exc_class = exc.Fault
 
-        if x is not None:
-            raise x
+        if exc_class is not None:
+            raise exc_class(msg, response=response)
 
         return response
 
@@ -229,3 +308,12 @@ class ApiV3(object):
         url = url.format(**kwargs)
         params = dict([(k, v) for k, v in kwargs.items() if not k in referenced])
         return self._request(url, params=params, method='PUT', check_for_errors=check_for_errors)
+
+    def delete(self, url, check_for_errors=True, **kwargs):
+        """
+        Performs a generic DELETE request for specified params, returning the response.
+        """
+        referenced = self._extract_referenced_vars(url)
+        url = url.format(**kwargs)
+        params = dict([(k, v) for k, v in kwargs.items() if not k in referenced])
+        return self._request(url, params=params, method='DELETE', check_for_errors=check_for_errors)
