@@ -1,9 +1,11 @@
-from __future__ import absolute_import, unicode_literals
+from __future__ import division, absolute_import, print_function, unicode_literals
 
-from stravalib import model, attributes, unithelper as uh
+from stravalib import model, attributes, exc, unithelper as uh
 from stravalib.client import Client
 from stravalib.tests.functional import FunctionalTestBase
 import datetime
+import requests
+
 
 class ClientTest(FunctionalTestBase):
     def test_get_starred_segment(self):
@@ -36,6 +38,24 @@ class ClientTest(FunctionalTestBase):
         # Ensure that iw as read in with correct units
         self.assertEquals(22.5308, float(uh.kilometers(activity.distance)))
 
+    def test_get_activity_and_segments(self):
+        """ Test include_all_efforts parameter on activity fetching. """
+        if not self.activity_id:
+            self.fail("Include an activity_id in test.ini to test segment_efforts")
+
+        activity = self.client.get_activity(self.activity_id, include_all_efforts=True)
+        self.assertTrue(isinstance(activity.segment_efforts, list))
+
+        # Check also when we have no parameters segment_efforts is None
+        activity_no_segments = self.client.get_activity(self.activity_id)
+        self.assertTrue(activity.segment_efforts, None)
+
+    def test_get_route(self):
+        route = self.client.get_route(3445913)
+
+        self.assertEquals('Baveno - Mottarone', route.name)
+        self.assertAlmostEquals(1265.20, float(uh.meters(route.elevation_gain)), 2)
+
     def test_get_activity_laps(self):
         activity = self.client.get_activity(165094211)
         laps = list(self.client.get_activity_laps(165094211))
@@ -50,7 +70,7 @@ class ClientTest(FunctionalTestBase):
         Test loading zones for activity.
         """
         zones = self.client.get_activity_zones(99895560)
-        print zones
+        print(zones)
         self.assertEquals(1, len(zones))
         self.assertIsInstance(zones[0], model.PaceActivityZone)
 
@@ -73,11 +93,11 @@ class ClientTest(FunctionalTestBase):
         """
         Test photos on activity
         """
-        activity = self.client.get_activity(152668627)
-        self.assertTrue(activity.photo_count > 0)
-        photos = list(activity.photos)
+        activity = self.client.get_activity(643026323)
+        self.assertTrue(activity.total_photo_count > 0)
+        photos = list(activity.full_photos)
         self.assertEqual(len(photos), 1)
-        self.assertEqual(len(photos), activity.photo_count)
+        self.assertEqual(len(photos), activity.total_photo_count)
         self.assertIsInstance(photos[0], model.ActivityPhoto)
 
     def test_activity_kudos(self):
@@ -114,7 +134,19 @@ class ClientTest(FunctionalTestBase):
         # latlng stream
         self.assertIsInstance(streams['latlng'].data, list)
         self.assertIsInstance(streams['latlng'].data[0][0], float)
-        
+
+    def test_route_streams(self):
+        """
+        Test toute streams
+        """
+        stypes = ['latlng', 'distance', 'altitude']
+
+        streams = self.client.get_route_streams(3445913)
+        self.assertEqual(len(streams.keys()), 3)
+
+        for t in stypes:
+            self.assertIn(t, streams.keys())
+
     def test_related_activities(self):
         """
         Test get_related_activities on an activity and related property of Activity
@@ -122,13 +154,13 @@ class ClientTest(FunctionalTestBase):
         activity_id = 152668627
         activity = self.client.get_activity(activity_id)
         related_activities = list(self.client.get_related_activities(activity_id))
-        
+
         # Check the number of related_activities matches what activity would expect
         self.assertEqual(len(related_activities), activity.athlete_count-1)
-        
+
         # Check the related property gives the same result
         related_activities_from_property = list(activity.related)
-        self.assertEqual(related_activities, related_activities_from_property)        
+        self.assertEqual(related_activities, related_activities_from_property)
 
     def test_effort_streams(self):
         """
@@ -149,32 +181,13 @@ class ClientTest(FunctionalTestBase):
         self.assertEqual(len(streams['distance'].data),
                          min(1000, streams['distance'].original_size))
 
-
     def test_get_curr_athlete(self):
         athlete = self.client.get_athlete()
 
-        print athlete
-        self.fail("break")
         # Just some basic sanity checks here
-        self.assertEquals('Jeff', athlete.firstname)
-        self.assertEquals('Remer', athlete.lastname)
-        self.assertEquals(3, len(athlete.clubs))
-        self.assertEquals('Team Roaring Mouse', athlete.clubs[0].name)
-        self.assertEquals(1, len(athlete.shoes))
-        print athlete.shoes
+        self.assertTrue(len(athlete.firstname) > 0)
 
-        self.assertIsInstance(athlete.shoes[0], model.Shoe)
-        self.assertIsInstance(athlete.clubs[0], model.Club)
-
-        self.assertIsInstance(athlete.bikes[0], model.Bike)
-
-        friends = list(athlete.friends)
-        self.assertGreater(len(friends), 1)
-        self.assertIsInstance(friends[0], model.Athlete)
-
-        followers = list(athlete.followers)
-        self.assertGreater(len(followers), 1)
-        self.assertIsInstance(followers[0], model.Athlete)
+        self.assertTrue(athlete.athlete_type in ["runner", "cyclist"])
 
     def test_get_athlete_clubs(self):
         clubs = self.client.get_athlete_clubs()
@@ -205,7 +218,7 @@ class ClientTest(FunctionalTestBase):
         print(lb.effort_count)
         print(lb.entry_count)
         for i,e in enumerate(lb):
-            print '{0}: {1}'.format(i, e)
+            print('{0}: {1}'.format(i, e))
 
         self.assertEquals(10, len(lb.entries)) # 10 top results
         self.assertIsInstance(lb.entries[0], model.SegmentLeaderboardEntry)
@@ -218,7 +231,7 @@ class ClientTest(FunctionalTestBase):
         self.assertEquals(lb[0].athlete_name, "{0} {1}".format(athlete.firstname, athlete.lastname))
 
         effort = lb[0].effort
-        print effort
+        print(effort)
         self.assertIsInstance(effort, model.SegmentEffort)
         self.assertEquals('Hawk Hill', effort.name)
 
@@ -229,7 +242,7 @@ class ClientTest(FunctionalTestBase):
     def test_get_segment(self):
         segment = self.client.get_segment(229781)
         self.assertIsInstance(segment, model.Segment)
-        print segment
+        print(segment)
         self.assertEquals('Hawk Hill', segment.name)
         self.assertAlmostEqual(2.68, float(uh.kilometers(segment.distance)), places=2)
 
@@ -242,17 +255,17 @@ class ClientTest(FunctionalTestBase):
         efforts = self.client.get_segment_efforts(4357415,
                                      start_date_local = "2012-12-23T00:00:00Z",
                                      end_date_local   = "2012-12-23T11:00:00Z",)
-        print efforts
+        print(efforts)
 
         i = 0
         for effort in efforts:
-            print effort
+            print(effort)
             self.assertEqual(4357415, effort.segment.id)
             self.assertIsInstance(effort, model.BaseEffort)
             effort_date = effort.start_date_local
             self.assertEqual(effort_date.strftime("%Y-%m-%d"), "2012-12-23")
             i+=1
-        print i
+        print(i)
 
         self.assertGreater(i, 2)
 
@@ -262,17 +275,17 @@ class ClientTest(FunctionalTestBase):
         efforts = self.client.get_segment_efforts(4357415,
                                         start_date_local = start_date,
                                         end_date_local = end_date,)
-        print efforts
+        print(efforts)
 
         i = 0
         for effort in efforts:
-            print effort
+            print(effort)
             self.assertEqual(4357415, effort.segment.id)
             self.assertIsInstance(effort, model.BaseEffort)
             effort_date = effort.start_date_local
             self.assertEqual(effort_date.strftime("%Y-%m-%d"), "2012-12-31")
             i+=1
-        print i
+        print(i)
 
         self.assertGreater(i, 2)
 
@@ -290,3 +303,75 @@ class ClientTest(FunctionalTestBase):
         # For some reason these don't follow the simple math rules one might expect (so we round to int)
         self.assertAlmostEqual(results[0].elev_difference, segment.elevation_high - segment.elevation_low, places=0)
 
+
+class AuthenticatedAthleteTest(FunctionalTestBase):
+    """
+    Tests the function is_authenticated_athlete in model.Athlete
+    """
+    def test_caching(self):
+        a = model.Athlete()
+        a._is_authenticated = "Not None"
+        self.assertEqual(a.is_authenticated_athlete(), "Not None")
+
+    def test_correct_athlete_returns_true(self):
+        a = self.client.get_athlete()
+        self.assertTrue(a.is_authenticated_athlete())
+
+    def test_detailed_resource_state_means_true(self):
+        a = model.Athlete()
+        a.resource_state = attributes.DETAILED
+        self.assertTrue(a.is_authenticated_athlete())
+
+    def test_correct_athlete_not_detailed_returns_true(self):
+        a = self.client.get_athlete()
+        a.resource_state = attributes.SUMMARY
+        # Now will have to do a look up for the authenticated athlete and check the ids match
+        self.assertTrue(a.is_authenticated_athlete())
+
+    def test_not_authenticated_athlete_is_false(self):
+        CAV_ID = 1353775
+        a = self.client.get_athlete(CAV_ID)
+        self.assertEqual(a.resource_state, attributes.SUMMARY)
+        self.assertFalse(a.is_authenticated_athlete())
+
+
+class AthleteStatsTest(FunctionalTestBase):
+    """
+    Tests the functionality for collecting athlete statistics
+    http://strava.github.io/api/v3/athlete/#stats
+    """
+    def test_basic_get_from_client(self):
+        stats = self.client.get_athlete_stats()
+        self.assertIsInstance(stats, model.AthleteStats)
+        self.assertIsInstance(stats.recent_ride_totals, model.ActivityTotals)
+        print("Biggest climb: {!r}".format(stats.biggest_climb_elevation_gain))
+
+        # Check biggest_climb_elevation_gain has been set
+        self.assertTrue(uh.meters(stats.biggest_climb_elevation_gain) >= uh.meters(0))
+
+    def test_get_from_client_with_authenticated_id(self):
+        athlete_id = self.client.get_athlete().id
+        stats = self.client.get_athlete_stats(athlete_id)
+        self.assertIsInstance(stats, model.AthleteStats)
+        # Check same as before
+        self.assertEqual(stats.biggest_climb_elevation_gain, self.client.get_athlete_stats().biggest_climb_elevation_gain)
+
+    def test_get_from_client_with_wrong_id(self):
+        CAV_ID = 1353775
+        # Currently raises a requests.exceptions.HTTPError, TODO: better error handling
+        self.assertRaises(requests.exceptions.HTTPError, self.client.get_athlete_stats, CAV_ID)
+
+    def test_athlete_stats_property_option(self):
+        a = self.client.get_athlete()
+        stats = a.stats
+        self.assertIsInstance(stats, model.AthleteStats)
+
+    def test_athlete_stats_cached(self):
+        a = self.client.get_athlete()
+        a._stats = "Not None"
+        stats = a.stats
+        self.assertEqual(stats, "Not None")
+
+    def test_athlete_property_not_authenticated(self):
+        with self.assertRaises(NotImplementedError):
+            cav = self.client.get_athlete(1353775)
